@@ -50,7 +50,6 @@
 #include "eggshell.h"
 
 #define URL_MATCH_CURSOR  (GDK_HAND2)
-#define SKEY_MATCH_CURSOR (GDK_HAND2)
 
 typedef struct
 {
@@ -151,8 +150,8 @@ static guint signals[LAST_SIGNAL];
 #define HOSTCHARS_CLASS "[-[:alnum:]]"
 #define HOST HOSTCHARS_CLASS "+(\\." HOSTCHARS_CLASS "+)*"
 #define PORT "(?:\\:[[:digit:]]{1,5})?"
-#define PATHCHARS_CLASS "[-[:alnum:]\\Q_$.+!*,;@&=?/~#%\\E]"
-#define PATHTERM_CLASS "[^\\Q]'.}>) \t\r\n,\"\\E]"
+#define PATHCHARS_CLASS "[-[:alnum:]\\Q_$.+!*,:;@&=?/~#%\\E]"
+#define PATHTERM_CLASS "[^\\Q]'.:}>) \t\r\n,\"\\E]"
 #define SCHEME "(?:news:|telnet:|nntp:|file:\\/|https?:|ftps?:|sftp:|webcal:)"
 #define USERPASS USERCHARS_CLASS "+(?:" PASSCHARS_CLASS "+)?"
 #define URLPATH   "(?:(/"PATHCHARS_CLASS"+(?:[(]"PATHCHARS_CLASS"*[)])*"PATHCHARS_CLASS"*)*"PATHTERM_CLASS")?"
@@ -175,18 +174,6 @@ static GRegex **url_regexes;
 static TerminalURLFlavour *url_regex_flavors;
 static guint n_url_regexes;
 
-#ifdef ENABLE_SKEY
-static const TerminalRegexPattern skey_regex_patterns[] = {
-  { "s/key [[:digit:]]* [-[:alnum:]]*",         FLAVOR_AS_IS },
-  { "otp-[a-z0-9]* [[:digit:]]* [-[:alnum:]]*", FLAVOR_AS_IS },
-};
-
-static GRegex **skey_regexes;
-static guint n_skey_regexes;
-
-static void  terminal_screen_skey_match_remove (TerminalScreen            *screen);
-#endif /* ENABLE_SKEY */
-
 G_DEFINE_TYPE (TerminalScreen, terminal_screen, VTE_TYPE_TERMINAL)
 
 static char *
@@ -313,6 +300,23 @@ terminal_screen_realize (GtkWidget *widget)
                                            !window_uses_argb_visual (screen));
 }
 
+#if GTK_CHECK_VERSION (2, 99, 0)
+
+static void
+terminal_screen_style_updated (GtkWidget *widget)
+{
+  TerminalScreen *screen = TERMINAL_SCREEN (widget);
+
+  GTK_WIDGET_CLASS (terminal_screen_parent_class)->style_updated (widget);
+
+  update_color_scheme (screen);
+
+  if (gtk_widget_get_realized (widget))
+    terminal_screen_change_font (screen);
+}
+
+#else /* GTK 2.0 */
+
 static void
 terminal_screen_style_set (GtkWidget *widget,
                            GtkStyle *previous_style)
@@ -329,6 +333,8 @@ terminal_screen_style_set (GtkWidget *widget,
     terminal_screen_change_font (screen);
 }
 
+#endif /* GTK 3.0 */
+
 #ifdef GNOME_ENABLE_DEBUG
 static void
 size_request (GtkWidget *widget,
@@ -511,7 +517,11 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   object_class->set_property = terminal_screen_set_property;
 
   widget_class->realize = terminal_screen_realize;
+#if GTK_CHECK_VERSION (2, 99, 0)
+  widget_class->style_updated = terminal_screen_style_updated;
+#else
   widget_class->style_set = terminal_screen_style_set;
+#endif
   widget_class->drag_data_received = terminal_screen_drag_data_received;
   widget_class->button_press_event = terminal_screen_button_press;
   widget_class->popup_menu = terminal_screen_popup_menu;
@@ -624,23 +634,6 @@ terminal_screen_class_init (TerminalScreenClass *klass)
       url_regex_flavors[i] = url_regex_patterns[i].flavor;
     }
 
-#ifdef ENABLE_SKEY
-  n_skey_regexes = G_N_ELEMENTS (skey_regex_patterns);
-  skey_regexes = g_new0 (GRegex*, n_skey_regexes);
-
-  for (i = 0; i < n_skey_regexes; ++i)
-    {
-      GError *error = NULL;
-
-      skey_regexes[i] = g_regex_new (skey_regex_patterns[i].pattern, G_REGEX_OPTIMIZE, 0, &error);
-      if (error)
-        {
-          g_message ("%s", error->message);
-          g_error_free (error);
-        }
-    }
-#endif /* ENABLE_SKEY */
-
   /* This fixes bug #329827 */
   app = terminal_app_get ();
   terminal_screen_class_enable_menu_bar_accel_notify_cb (app, NULL, klass);
@@ -979,32 +972,6 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
       vte_terminal_set_scrollback_lines (vte_terminal, lines);
     }
 
-#ifdef ENABLE_SKEY
-  if (!prop_name || prop_name == I_(TERMINAL_PROFILE_USE_SKEY))
-    {
-      if (terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_USE_SKEY))
-        {
-          guint i;
-
-          for (i = 0; i < n_skey_regexes; ++i)
-            {
-              TagData *tag_data;
-
-              tag_data = g_slice_new (TagData);
-              tag_data->flavor = FLAVOR_SKEY;
-              tag_data->tag = vte_terminal_match_add_gregex (vte_terminal, skey_regexes[i], 0);
-              vte_terminal_match_set_cursor_type (vte_terminal, tag_data->tag, SKEY_MATCH_CURSOR);
-
-              priv->match_tags = g_slist_prepend (priv->match_tags, tag_data);
-            }
-        }
-      else
-        {
-          terminal_screen_skey_match_remove (screen);
-        }
-    }
-#endif /* ENABLE_SKEY */
-
   if (!prop_name ||
       prop_name == I_(TERMINAL_PROFILE_BACKGROUND_TYPE) ||
       prop_name == I_(TERMINAL_PROFILE_BACKGROUND_IMAGE) ||
@@ -1069,16 +1036,40 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
   g_object_thaw_notify (object);
 }
 
+#if GTK_CHECK_VERSION (2, 99, 0)
+static void
+rgba_to_color (GdkColor *color,
+               const GdkRGBA *rgba)
+{
+        color->red = rgba->red * 65535.;
+        color->green = rgba->green * 65535.;
+        color->blue = rgba->blue * 65535.;
+        color->pixel = 0;
+}
+#endif
+
 static void
 update_color_scheme (TerminalScreen *screen)
 {
   TerminalScreenPrivate *priv = screen->priv;
   TerminalProfile *profile = priv->profile;
-  GtkStyle *style;
   GdkColor colors[TERMINAL_PALETTE_SIZE];
   const GdkColor *fg_color, *bg_color, *bold_color;
   GdkColor fg, bg;
   guint n_colors;
+#if GTK_CHECK_VERSION (2, 99, 0)
+  GtkStyleContext *context;
+  GdkRGBA rgba;
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (screen));
+  gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &rgba);
+  rgba_to_color (&fg, &rgba);
+
+  gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &rgba);
+  rgba_to_color (&bg, &rgba);
+
+#else /* GTK 2.0 */
+  GtkStyle *style;
 
   style = gtk_widget_get_style (GTK_WIDGET (screen));
   if (!style)
@@ -1086,6 +1077,8 @@ update_color_scheme (TerminalScreen *screen)
 
   fg = style->text[GTK_STATE_NORMAL];
   bg = style->base[GTK_STATE_NORMAL];
+#endif /* GTK 3.0 */
+
   bold_color = NULL;
 
   if (!terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_USE_THEME_COLORS))
@@ -1392,7 +1385,13 @@ get_child_environment (TerminalScreen *screen,
   
 #ifdef GDK_WINDOWING_X11
   /* FIXME: moving the tab between windows, or the window between displays will make the next two invalid... */
-  g_hash_table_replace (env_table, g_strdup ("WINDOWID"), g_strdup_printf ("%ld", GDK_WINDOW_XWINDOW (gtk_widget_get_window (window))));
+  g_hash_table_replace (env_table, g_strdup ("WINDOWID"), 
+                        g_strdup_printf ("%ld", 
+#if GTK_CHECK_VERSION (2, 91, 6)
+                                         GDK_WINDOW_XID (gtk_widget_get_window (window))));
+#else
+                                         GDK_WINDOW_XWINDOW (gtk_widget_get_window (window))));
+#endif
   g_hash_table_replace (env_table, g_strdup ("DISPLAY"), g_strdup (gdk_display_get_name (gtk_widget_get_display (window))));
 #endif
 
@@ -1612,29 +1611,23 @@ terminal_screen_button_press (GtkWidget      *widget,
 
   /* FIXMEchpe: add vte API to do this check by widget coords instead of grid coords */
   matched_string = terminal_screen_check_match (screen, row, col, &matched_flavor);
-  
+
   if (matched_string != NULL &&
       (event->button == 1 || event->button == 2) &&
       (state & GDK_CONTROL_MASK))
     {
       gboolean handled = FALSE;
 
-#ifdef ENABLE_SKEY
-      if (matched_flavor != FLAVOR_SKEY ||
-          terminal_profile_get_property_boolean (screen->priv->profile, TERMINAL_PROFILE_USE_SKEY))
-#endif
+      g_signal_emit (screen, signals[MATCH_CLICKED], 0,
+                     matched_string,
+                     matched_flavor,
+                     state,
+                     &handled);
+      if (handled)
         {
-          g_signal_emit (screen, signals[MATCH_CLICKED], 0,
-                         matched_string,
-                         matched_flavor,
-                         state,
-                         &handled);
-        }
-
-        g_free (matched_string);
-
-        if (handled)
+          g_free (matched_string);
           return TRUE; /* don't do anything else such as select with the click */
+        }
     }
 
   if (event->button == 3 &&
@@ -1655,6 +1648,8 @@ terminal_screen_button_press (GtkWidget      *widget,
       return TRUE;
     }
 
+  g_free (matched_string);
+
   /* default behavior is to let the terminal widget deal with it */
   if (button_press_event)
     return button_press_event (widget, event);
@@ -2211,30 +2206,6 @@ terminal_screen_get_cell_size (TerminalScreen *screen,
   *cell_height_pixels = vte_terminal_get_char_height (terminal);
 }
 
-#ifdef ENABLE_SKEY
-static void
-terminal_screen_skey_match_remove (TerminalScreen *screen)
-{
-  TerminalScreenPrivate *priv = screen->priv;
-  GSList *l, *next;
-
-  l = priv->match_tags;
-  while (l != NULL)
-    {
-      TagData *tag_data = (TagData *) l->data;
-
-      next = l->next;
-      if (tag_data->flavor == FLAVOR_SKEY)
-        {
-          vte_terminal_match_remove (VTE_TERMINAL (screen), tag_data->tag);
-          priv->match_tags = g_slist_delete_link (priv->match_tags, l);
-        }
-
-      l = next;
-    }
-}
-#endif /* ENABLE_SKEY */
-
 static char*
 terminal_screen_check_match (TerminalScreen *screen,
 			     int        column,
